/*
 * Oscar Shell Service
 * Copyright (c) 2004, Richard S. Hall
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the ungoverned.org nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Richard S. Hall (heavy@ungoverned.org)
 * Contributor(s):
 *
**/
package org.ungoverned.osgi.bundle.shell;

import java.io.PrintStream;
import java.security.AccessController;
import java.security.PrivilegedActionException;
import java.security.PrivilegedExceptionAction;
import java.util.HashMap;
import java.util.Set;
import java.util.TreeMap;

import org.osgi.framework.*;
import org.ungoverned.osgi.service.shell.Command;
import org.ungoverned.osgi.service.shell.ShellService;

public class Activator implements BundleActivator
{
    private transient BundleContext m_context = null;
    private transient ShellServiceImpl m_shell = null;

    // Reusable admin permission object for all instances
    // of the BundleImpl.
    private static AdminPermission m_adminPerm = new AdminPermission();

    public void start(BundleContext context)
    {
        m_context = context;

        // Register shell service implementation.
        context.registerService(
            org.ungoverned.osgi.service.shell.ShellService.class.getName(),
            m_shell = new ShellServiceImpl(), null);

        // Listen for registering/unregistering of shell command
        // services so that we can automatically add/remove them
        // from our list of available commands.
        ServiceListener sl = new ServiceListener() {
            public void serviceChanged(ServiceEvent event)
            {
                if (event.getType() == ServiceEvent.REGISTERED)
                {
                    m_shell.addCommand(event.getServiceReference());
                }
                else if (event.getType() == ServiceEvent.UNREGISTERING)
                {
                    m_shell.removeCommand(event.getServiceReference());
                }
                else
                {
                }
            }
        };

        try
        {
            m_context.addServiceListener(sl,
                "(objectClass=" + org.ungoverned.osgi.service.shell.Command.class.getName()
                + ")");
        }
        catch (InvalidSyntaxException ex)
        {
            System.err.println("ShellServiceActivator: Cannot register service listener.");
            System.err.println("ShellServiceActivator: " + ex);
        }

        // Now manually try to find any commands that have already
        // been registered (i.e., we didn't see their service events).
        initializeCommands();

        // Register "exports" command service.
        context.registerService(
            org.ungoverned.osgi.service.shell.Command.class.getName(),
            new BundleLevelCommandImpl(m_context), null);

        // Register "cd" command service.
        String[] classes = new String[2];
        classes[0] = org.ungoverned.osgi.service.shell.Command.class.getName();
        classes[1] = org.ungoverned.osgi.service.shell.CdCommand.class.getName();
        context.registerService(
            classes, new CdCommandImpl(m_context), null);

        // Register "exports" command service.
        context.registerService(
            org.ungoverned.osgi.service.shell.Command.class.getName(),
            new PackagesCommandImpl(m_context), null);

        // Register "headers" command service.
        context.registerService(
            org.ungoverned.osgi.service.shell.Command.class.getName(),
            new HeadersCommandImpl(m_context), null);

        // Register "help" command service.
        context.registerService(
            org.ungoverned.osgi.service.shell.Command.class.getName(),
            new HelpCommandImpl(m_context), null);

        // Register "install" command service.
        context.registerService(
            org.ungoverned.osgi.service.shell.Command.class.getName(),
            new InstallCommandImpl(m_context), null);

        // Register "ps" command service.
        context.registerService(
            org.ungoverned.osgi.service.shell.Command.class.getName(),
            new PsCommandImpl(m_context), null);

        // Register "refresh" command service.
        context.registerService(
            org.ungoverned.osgi.service.shell.Command.class.getName(),
            new RefreshCommandImpl(m_context), null);

        // Register "services" command service.
        context.registerService(
            org.ungoverned.osgi.service.shell.Command.class.getName(),
            new ServicesCommandImpl(m_context), null);

        // Register "startlevel" command service.
        context.registerService(
            org.ungoverned.osgi.service.shell.Command.class.getName(),
            new StartLevelCommandImpl(m_context), null);

        // Register "shutdown" command service.
        context.registerService(
            org.ungoverned.osgi.service.shell.Command.class.getName(),
            new ShutdownCommandImpl(m_context), null);

        // Register "start" command service.
        context.registerService(
            org.ungoverned.osgi.service.shell.Command.class.getName(),
            new StartCommandImpl(m_context), null);

        // Register "stop" command service.
        context.registerService(
            org.ungoverned.osgi.service.shell.Command.class.getName(),
            new StopCommandImpl(m_context), null);

        // Register "uninstall" command service.
        context.registerService(
            org.ungoverned.osgi.service.shell.Command.class.getName(),
            new UninstallCommandImpl(m_context), null);

        // Register "update" command service.
        context.registerService(
            org.ungoverned.osgi.service.shell.Command.class.getName(),
            new UpdateCommandImpl(m_context), null);

        // Register "version" command service.
        context.registerService(
            org.ungoverned.osgi.service.shell.Command.class.getName(),
            new VersionCommandImpl(m_context), null);
    }

    public void stop(BundleContext context)
    {
        m_shell.clearCommands();
    }

    private void initializeCommands()
    {
        synchronized (m_shell)
        {
            try
            {
                ServiceReference[] refs = m_context.getServiceReferences(
                    org.ungoverned.osgi.service.shell.Command.class.getName(), null);
                if (refs != null)
                {
                    for (int i = 0; i < refs.length; i++)
                    {
                        m_shell.addCommand(refs[i]);
                    }
                }
            }
            catch (Exception ex)
            {
                System.err.println("ShellServiceActivator: " + ex);
            }
        }
    }

    private class ShellServiceImpl implements ShellService
    {
        private HashMap m_commandRefMap = new HashMap();
        private TreeMap m_commandNameMap = new TreeMap();
        private ExecutePrivileged m_executePriv = null;

        public synchronized String[] getCommands()
        {
            Set ks = m_commandNameMap.keySet();
            String[] cmds = (ks == null)
                ? new String[0] : (String[]) ks.toArray(new String[ks.size()]);
            return cmds;
        }

        public synchronized String getCommandUsage(String name)
        {
            CommandRecord cr = (CommandRecord) m_commandNameMap.get(name);
            return (cr == null) ? null : cr.m_command.getUsage();
        }

        public synchronized String getCommandDescription(String name)
        {
            CommandRecord cr = (CommandRecord) m_commandNameMap.get(name);
            return (cr == null) ? null : cr.m_command.getShortDescription();
        }

        public synchronized ServiceReference getCommandReference(String name)
        {
            return (ServiceReference) m_commandNameMap.get(name);
        }

        public synchronized void removeCommand(ServiceReference ref)
        {
            CommandRecord cr = (CommandRecord) m_commandRefMap.remove(ref);
            if (cr != null)
            {
                m_commandNameMap.remove(cr.m_command.getName());
            }
        }

        public synchronized void executeCommand(
            String commandLine, PrintStream out, PrintStream err) throws Exception
        {
            commandLine = commandLine.trim();
            String commandName = (commandLine.indexOf(' ') >= 0)
                ? commandLine.substring(0, commandLine.indexOf(' ')) : commandLine;
            Command command = getCommand(commandName);
            if (command != null)
            {
                if (System.getSecurityManager() != null)
                {
                    try
                    {
                        AccessController.doPrivileged(
                            new ExecutePrivileged(command, commandLine, out, err));
                    }
                    catch (PrivilegedActionException ex)
                    {
                        throw ex.getException();
                    }
                }
                else
                {
                    try
                    {
                        command.execute(commandLine, out, err);
                    }
                    catch (Throwable ex)
                    {
                        err.println("Unable to execute command: " + ex);
                        ex.printStackTrace(err);
                    }
                }
            }
            else
            {
                err.println("Command not found.");
            }
        }

        protected synchronized Command getCommand(String name)
        {
            CommandRecord cr = (CommandRecord) m_commandNameMap.get(name);
            return (cr == null) ? null : cr.m_command;
        }

        protected synchronized void addCommand(ServiceReference ref)
        {
            CommandRecord cr = new CommandRecord(ref);
            m_commandRefMap.put(ref, cr);
            m_commandNameMap.put(cr.m_command.getName(), cr);
        }

        protected synchronized void clearCommands()
        {
            m_commandRefMap.clear();
            m_commandNameMap.clear();
        }
    }

    private class CommandRecord
    {
        public ServiceReference m_ref = null;
        public Command m_command = null;

        public CommandRecord(ServiceReference ref)
        {
            m_ref = ref;
            m_command = (Command) m_context.getService(ref);
        }
    }

    public static class ExecutePrivileged implements PrivilegedExceptionAction
    {
        private Command m_command = null;
        private String m_commandLine = null;
        private PrintStream m_out = null;
        private PrintStream m_err = null;

        public ExecutePrivileged(
            Command command, String commandLine,
            PrintStream out, PrintStream err)
            throws Exception
        {
            m_command = command;
            m_commandLine = commandLine;
            m_out = out;
            m_err = err;
        }

        public Object run() throws Exception
        {
            m_command.execute(m_commandLine, m_out, m_err);
            return null;
        }
    }
}
