/*
 * Oscar Shell Service
 * Copyright (c) 2004, Richard S. Hall
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the ungoverned.org nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Richard S. Hall (heavy@ungoverned.org)
 * Contributor(s):
 *
**/
package org.ungoverned.osgi.bundle.shell;

import java.io.PrintStream;
import java.util.StringTokenizer;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.service.startlevel.StartLevel;
import org.ungoverned.osgi.service.shell.Command;

public class BundleLevelCommandImpl implements Command
{
    private BundleContext m_context = null;

    public BundleLevelCommandImpl(BundleContext context)
    {
        m_context = context;
    }

    public String getName()
    {
        return "bundlelevel";
    }

    public String getUsage()
    {
        return "bundlelevel <level> <id> ... | <id>";
    }

    public String getShortDescription()
    {
        return "set or get bundle start level.";
    }

    public void execute(String s, PrintStream out, PrintStream err)
    {
        // Get start level service.
        ServiceReference ref = m_context.getServiceReference(
            org.osgi.service.startlevel.StartLevel.class.getName());
        if (ref == null)
        {
            out.println("StartLevel service is unavailable.");
            return;
        }

        StartLevel sl = (StartLevel) m_context.getService(ref);
        if (sl == null)
        {
            out.println("StartLevel service is unavailable.");
            return;
        }

        // Parse command line.
        StringTokenizer st = new StringTokenizer(s, " ");

        // Ignore the command name.
        st.nextToken();

        // If there is only one token, then assume it is
        // a bundle ID for which we must retrieve the bundle
        // level.
        if (st.countTokens() == 1)
        {
            // Get the bundle and display start level.
            Bundle bundle = null;
            String token = null;
            try
            {
                token = st.nextToken();
                long id = Long.parseLong(token);
                bundle = m_context.getBundle(id);
                if (bundle != null)
                {
                    out.println("Bundle " + token + " is level "
                        + sl.getBundleStartLevel(bundle));
                }
                else
                {
                    err.println("Bundle ID " + token + " is invalid.");
                }
            }
            catch (NumberFormatException ex)
            {
                err.println("Unable to parse integer '" + token + "'.");
            }
            catch (Exception ex)
            {
                err.println(ex.toString());
            }
        }
        // If there is more than one token, assume the first
        // token is the new start level and the remaining
        // tokens are the bundle IDs whose start levels should
        // be changed.
        else if (st.countTokens() > 1)
        {
            // Get the bundle.
            Bundle bundle = null;
            String token = null;
            int startLevel = -1;

            try
            {
                token = st.nextToken();
                startLevel = Integer.parseInt(token);
            }
            catch (NumberFormatException ex)
            {
                err.println("Unable to parse start level '" + token + "'.");
            }

            // Ignore invalid start levels.
            if (startLevel > 0)
            {
                // Set the start level for each specified bundle.
                while (st.hasMoreTokens())
                {
                    try
                    {
                        token = st.nextToken();
                        long id = Long.parseLong(token);
                        bundle = m_context.getBundle(id);
                        if (bundle != null)
                        {
                            sl.setBundleStartLevel(bundle, startLevel);
                        }
                        else
                        {
                            err.println("Bundle ID '" + token + "' is invalid.");
                        }
                    }
                    catch (NumberFormatException ex)
                    {
                        err.println("Unable to parse bundle ID '" + token + "'.");
                    }
                    catch (Exception ex)
                    {
                        err.println(ex.toString());
                    }
                }
            }
            else
            {
                err.println("Invalid start level.");
            }
        }
        else
        {
            err.println("Incorrect number of arguments.");
        }
    }
}