/*
 * Oscar Shell Service
 * Copyright (c) 2004, Richard S. Hall
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the ungoverned.org nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Richard S. Hall (heavy@ungoverned.org)
 * Contributor(s):
 *
**/
package org.ungoverned.osgi.bundle.shell;

import java.io.PrintStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.StringTokenizer;

import org.osgi.framework.*;
import org.ungoverned.osgi.service.shell.CdCommand;
import org.ungoverned.osgi.service.shell.Command;

public class InstallCommandImpl implements Command
{
    private BundleContext m_context = null;

    public InstallCommandImpl(BundleContext context)
    {
        m_context = context;
    }

    public String getName()
    {
        return "install";
    }

    public String getUsage()
    {
        return "install <URL> [<URL> ...]";
    }

    public String getShortDescription()
    {
        return "install bundle(s).";
    }

    public void execute(String s, PrintStream out, PrintStream err)
    {
        StringTokenizer st = new StringTokenizer(s, " ");

        // Ignore the command name.
        st.nextToken();

        // There should be at least one URL.
        if (st.countTokens() >= 1)
        {
            while (st.hasMoreTokens())
            {
                String location = st.nextToken().trim();
                install(location, out, err);
            }
        }
        else
        {
            err.println("Incorrect number of arguments");
        }
    }

    protected Bundle install(String location, PrintStream out, PrintStream err)
    {
        String abs = absoluteLocation(location);
        if (abs == null)
        {
            err.println("Malformed location: " + location);
        }
        else
        {
            try {
                return m_context.installBundle(abs, null);
            } catch (IllegalStateException ex) {
                err.println(ex.toString());
            } catch (BundleException ex) {
                if (ex.getNestedException() != null)
                {
                    err.println(ex.getNestedException().toString());
                }
                else
                {
                    err.println(ex.toString());
                }
            } catch (Exception ex) {
                err.println(ex.toString());
            }
        }
        return null;
    }

    private String absoluteLocation(String location)
    {
        if (!location.endsWith(".jar"))
        {
            location = location + ".jar";
        }
        try {
            URL url = new URL(location);
        } catch (MalformedURLException ex) {
            // Try to create a valid URL using the base URL
            // contained in the "cd" command service.
            String baseURL = "";

            try {
                // Get a reference to the "cd" command service.
                ServiceReference ref = m_context.getServiceReference(
                    org.ungoverned.osgi.service.shell.CdCommand.class.getName());

                if (ref != null)
                {
                    CdCommand cd = (CdCommand) m_context.getService(ref);
                    baseURL = cd.getBaseURL();
                    baseURL = (baseURL == null) ? "" : baseURL;
                    m_context.ungetService(ref);
                }

                String theURL = baseURL + location;
                URL url = new URL(theURL);

            } catch (Exception ex2) {
                return null;
            }
            location = baseURL + location;
        }
        return location;
    }
}