/*
 * Oscar Shell Service
 * Copyright (c) 2004, Richard S. Hall
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the ungoverned.org nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Richard S. Hall (heavy@ungoverned.org)
 * Contributor(s):
 *
**/
package org.ungoverned.osgi.bundle.shell;

import java.io.PrintStream;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.ungoverned.osgi.service.shell.Command;

public class ServicesCommandImpl implements Command
{
    private static final String IN_USE_SWITCH = "-u";
    private static final String SHOW_ALL_SWITCH = "-a";

    private BundleContext m_context = null;

    public ServicesCommandImpl(BundleContext context)
    {
        m_context = context;
    }

    public String getName()
    {
        return "services";
    }

    public String getUsage()
    {
        return "services [-u] [-a] [<id> ...]";
    }

    public String getShortDescription()
    {
        return "list registered or used services.";
    }

    public void execute(String s, PrintStream out, PrintStream err)
    {
        StringTokenizer st = new StringTokenizer(s, " ");

        // Ignore the command name.
        st.nextToken();

        // Put the remaining tokens into a list.
        List tokens = new ArrayList();
        for (int i = 0; st.hasMoreTokens(); i++)
        {
            tokens.add(st.nextToken());
        }

        // Default switch values.
        boolean inUse = false;
        boolean showAll = false;

        // Check for "in use" switch.
        if (tokens.contains(IN_USE_SWITCH))
        {
            // Remove the switch and set boolean flag.
            tokens.remove(IN_USE_SWITCH);
            inUse = true;
        }

        // Check for "show all" switch.
        if (tokens.contains(SHOW_ALL_SWITCH))
        {
            // Remove the switch and set boolean flag.
            tokens.remove(SHOW_ALL_SWITCH);
            showAll = true;
        }

        // If there are bundle IDs specified then print their
        // services and associated service properties, otherwise
        // list all bundles and services.
        if (tokens.size() >= 1)
        {
            while (tokens.size() > 0)
            {
                String id = (String) tokens.remove(0);

                boolean headerPrinted = false;
                boolean needSeparator = false;

                try
                {
                    long l = Long.parseLong(id);
                    Bundle bundle = m_context.getBundle(l);
                    if (bundle != null)
                    {
                        ServiceReference[] refs = null;
                        
                        // Get registered or in-use services.
                        if (inUse)
                        {
                            refs = bundle.getServicesInUse();
                        }
                        else
                        {
                            refs = bundle.getRegisteredServices();
                        }

                        // Print properties for each service.
                        for (int refIdx = 0;
                            (refs != null) && (refIdx < refs.length);
                            refIdx++)
                        {
                            String[] objectClass = (String[])
                                refs[refIdx].getProperty("objectClass");

                            // Determine if we need to print the service, depending
                            // on whether it is a command service or not.
                            boolean print = true;
                            for (int ocIdx = 0;
                                !showAll && (ocIdx < objectClass.length);
                                ocIdx++)
                            {
                                if (objectClass[ocIdx].equals(
                                    org.ungoverned.osgi.service.shell.Command.class.getName()))
                                {
                                    print = false;
                                }
                            }

                            // Print header if we have not already done so.
                            if (!headerPrinted)
                            {
                                headerPrinted = true;
                                String title = Util.getBundleName(bundle);
                                title = (inUse)
                                    ? title + " uses:"
                                    : title + " provides:";
                                out.println("");
                                out.println(title);
                                out.println(Util.getUnderlineString(title));
                            }

                            if (showAll || print)
                            {
                                // Print service separator if necessary.
                                if (needSeparator)
                                {
                                    out.println("----");
                                }

                                // Print service properties.
                                String[] keys = refs[refIdx].getPropertyKeys();
                                for (int keyIdx = 0;
                                    (keys != null) && (keyIdx < keys.length);
                                    keyIdx++)
                                {
                                    Object v = refs[refIdx].getProperty(keys[keyIdx]);
                                    out.println(
                                        keys[keyIdx] + " = " + Util.getValueString(v));
                                }
                                
                                needSeparator = true;
                            }
                        }
                    }
                    else
                    {
                        err.println("Bundle ID " + id + " is invalid.");
                    }
                }
                catch (NumberFormatException ex)
                {
                    err.println("Unable to parse id '" + id + "'.");
                }
                catch (Exception ex)
                {
                    err.println(ex.toString());
                }
            }
        }
        else
        {
            Bundle[] bundles = m_context.getBundles();
            if (bundles != null)
            {
                // TODO: Sort list.
                for (int bundleIdx = 0; bundleIdx < bundles.length; bundleIdx++)
                {
                    boolean headerPrinted = false;
                    ServiceReference[] refs = null;

                    // Get registered or in-use services.
                    if (inUse)
                    {
                        refs = bundles[bundleIdx].getServicesInUse();
                    }
                    else
                    {
                        refs = bundles[bundleIdx].getRegisteredServices();
                    }

                    for (int refIdx = 0; (refs != null) && (refIdx < refs.length); refIdx++)
                    { 
                        String[] objectClass = (String[])
                            refs[refIdx].getProperty("objectClass");

                        // Determine if we need to print the service, depending
                        // on whether it is a command service or not.
                        boolean print = true;
                        for (int ocIdx = 0;
                            !showAll && (ocIdx < objectClass.length);
                            ocIdx++)
                        {
                            if (objectClass[ocIdx].equals(
                                org.ungoverned.osgi.service.shell.Command.class.getName()))
                            {
                                print = false;
                            }
                        }

                        // Print the service if necessary.
                        if (showAll || print)
                        {
                            if (!headerPrinted)
                            {
                                headerPrinted = true;
                                String title = Util.getBundleName(bundles[bundleIdx]);
                                title = (inUse)
                                    ? title + " uses:"
                                    : title + " provides:";
                                out.println("\n" + title);
                                out.println(Util.getUnderlineString(title));
                            }
                            out.println(Util.getValueString(objectClass));
                        }
                    }
                }
            }
            else
            {
                out.println("There are no registered services.");
            }
        }
    }
}
