/*
 * Oscar Shell Service
 * Copyright (c) 2004, Richard S. Hall
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the ungoverned.org nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Richard S. Hall (heavy@ungoverned.org)
 * Contributor(s):
 *
**/
package org.ungoverned.osgi.bundle.shell;

import java.io.IOException;
import java.io.InputStream;
import java.io.PrintStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.StringTokenizer;

import org.osgi.framework.*;
import org.ungoverned.osgi.service.shell.CdCommand;
import org.ungoverned.osgi.service.shell.Command;

public class UpdateCommandImpl implements Command
{
    private BundleContext m_context = null;

    public UpdateCommandImpl(BundleContext context)
    {
        m_context = context;
    }

    public String getName()
    {
        return "update";
    }

    public String getUsage()
    {
        return "update <id> [<URL>]";
    }

    public String getShortDescription()
    {
        return "update bundle.";
    }

    public void execute(String s, PrintStream out, PrintStream err)
    {
        StringTokenizer st = new StringTokenizer(s, " ");

        // Ignore the command name.
        st.nextToken();

        // There should be at least a bundle ID, but there may
        // also be a URL.
        if ((st.countTokens() == 1) || (st.countTokens() == 2))
        {
            String id = st.nextToken().trim();
            String location = st.countTokens() == 0 ? null : st.nextToken().trim();

            if (location != null)
            {
                location = absoluteLocation(location);

                if (location == null)
                {
                    err.println("Malformed location: " + location);
                }
            }

            try
            {
                // Get the bundle id.
                long l = Long.parseLong(id);

                // Get the bundle.
                Bundle bundle = m_context.getBundle(l);
                if (bundle != null)
                {
                    // Create input stream from location if present
                    // and use it to update, otherwise just update.
                    if (location != null)
                    {
                        InputStream is = new URL(location).openStream();
                        bundle.update(is);
                    }
                    else
                    {
                        bundle.update();
                    }
                }
                else
                {
                    err.println("Bundle ID " + id + " is invalid.");
                }
            }
            catch (NumberFormatException ex)
            {
                err.println("Unable to parse id '" + id + "'.");
            }
            catch (MalformedURLException ex)
            {
                err.println("Unable to parse URL.");
            }
            catch (IOException ex)
            {
                err.println("Unable to open input stream: " + ex);
            }
            catch (BundleException ex)
            {
                if (ex.getNestedException() != null)
                {
                    err.println(ex.getNestedException().toString());
                }
                else
                {
                    err.println(ex.toString());
                }
            }
            catch (Exception ex)
            {
                err.println(ex.toString());
            }
        }
        else
        {
            err.println("Incorrect number of arguments");
        }
    }

    private String absoluteLocation(String location)
    {
        if (!location.endsWith(".jar"))
        {
            location = location + ".jar";
        }
        try {
            URL url = new URL(location);
        } catch (MalformedURLException ex) {
            // Try to create a valid URL using the base URL
            // contained in the "cd" command service.
            String baseURL = "";

            try {
                // Get a reference to the "cd" command service.
                ServiceReference ref = m_context.getServiceReference(
                    org.ungoverned.osgi.service.shell.CdCommand.class.getName());

                if (ref != null)
                {
                    CdCommand cd = (CdCommand) m_context.getService(ref);
                    baseURL = cd.getBaseURL();
                    baseURL = (baseURL == null) ? "" : baseURL;
                    m_context.ungetService(ref);
                }

                String theURL = baseURL + location;
                URL url = new URL(theURL);

            } catch (Exception ex2) {
                return null;
            }
            location = baseURL + location;
        }
        return location;
    }
}